@@ -1,3 +1,6 @@
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
 import sys
 
 if sys.version_info >= (2, 7):
