@@ -15,6 +15,7 @@ from six import StringIO
 from six import text_type
 
 from .. import mock
+from ..helpers import BUSYBOX_IMAGE_WITH_TAG
 from .testcases import docker_client
 from .testcases import DockerClientTestCase
 from .testcases import get_links
@@ -37,6 +38,8 @@ from compose.container import Container
 from compose.errors import OperationFailedError
 from compose.parallel import ParallelStreamWriter
 from compose.project import OneOffFilter
+from compose.project import Project
+from compose.service import BuildAction
 from compose.service import ConvergencePlan
 from compose.service import ConvergenceStrategy
 from compose.service import NetworkMode
@@ -67,7 +70,7 @@ class ServiceTest(DockerClientTestCase):
         create_and_start_container(foo)
 
         assert len(foo.containers()) == 1
-        assert foo.containers()[0].name == 'composetest_foo_1'
+        assert foo.containers()[0].name.startswith('composetest_foo_')
         assert len(bar.containers()) == 0
 
         create_and_start_container(bar)
@@ -77,8 +80,8 @@ class ServiceTest(DockerClientTestCase):
         assert len(bar.containers()) == 2
 
         names = [c.name for c in bar.containers()]
-        assert 'composetest_bar_1' in names
-        assert 'composetest_bar_2' in names
+        assert len(names) == 2
+        assert all(name.startswith('composetest_bar_') for name in names)
 
     def test_containers_one_off(self):
         db = self.create_service('db')
@@ -89,18 +92,18 @@ class ServiceTest(DockerClientTestCase):
     def test_project_is_added_to_container_name(self):
         service = self.create_service('web')
         create_and_start_container(service)
-        assert service.containers()[0].name == 'composetest_web_1'
+        assert service.containers()[0].name.startswith('composetest_web_')
 
     def test_create_container_with_one_off(self):
         db = self.create_service('db')
         container = db.create_container(one_off=True)
-        assert container.name == 'composetest_db_run_1'
+        assert container.name.startswith('composetest_db_run_')
 
     def test_create_container_with_one_off_when_existing_container_is_running(self):
         db = self.create_service('db')
         db.start()
         container = db.create_container(one_off=True)
-        assert container.name == 'composetest_db_run_1'
+        assert container.name.startswith('composetest_db_run_')
 
     def test_create_container_with_unspecified_volume(self):
         service = self.create_service('db', volumes=[VolumeSpec.parse('/var/db')])
@@ -373,7 +376,7 @@ class ServiceTest(DockerClientTestCase):
         self.client.create_volume(volume_name)
         service = Service('db', client=client, volumes=[
             MountSpec(type='volume', source=volume_name, target=container_path)
-        ], image='busybox:latest', command=['top'], project='composetest')
+        ], image=BUSYBOX_IMAGE_WITH_TAG, command=['top'], project='composetest')
         container = service.create_container()
         service.start_container(container)
         mount = container.get_mount(container_path)
@@ -388,7 +391,7 @@ class ServiceTest(DockerClientTestCase):
         container_path = '/container-tmpfs'
         service = Service('db', client=client, volumes=[
             MountSpec(type='tmpfs', target=container_path)
-        ], image='busybox:latest', command=['top'], project='composetest')
+        ], image=BUSYBOX_IMAGE_WITH_TAG, command=['top'], project='composetest')
         container = service.create_container()
         service.start_container(container)
         mount = container.get_mount(container_path)
@@ -424,6 +427,22 @@ class ServiceTest(DockerClientTestCase):
         new_container = service.recreate_container(old_container)
         assert new_container.get_mount('/data')['Source'] == volume_path
 
+    def test_recreate_volume_to_mount(self):
+        # https://github.com/docker/compose/issues/6280
+        service = Service(
+            project='composetest',
+            name='db',
+            client=self.client,
+            build={'context': 'tests/fixtures/dockerfile-with-volume'},
+            volumes=[MountSpec.parse({
+                'type': 'volume',
+                'target': '/data',
+            })]
+        )
+        old_container = create_and_start_container(service)
+        new_container = service.recreate_container(old_container)
+        assert new_container.get_mount('/data')['Source']
+
     def test_duplicate_volume_trailing_slash(self):
         """
         When an image specifies a volume, and the Compose file specifies a host path
@@ -458,7 +477,7 @@ class ServiceTest(DockerClientTestCase):
         volume_container_1 = volume_service.create_container()
         volume_container_2 = Container.create(
             self.client,
-            image='busybox:latest',
+            image=BUSYBOX_IMAGE_WITH_TAG,
             command=["top"],
             labels={LABEL_PROJECT: 'composetest'},
             host_config={},
@@ -489,7 +508,7 @@ class ServiceTest(DockerClientTestCase):
         assert old_container.get('Config.Entrypoint') == ['top']
         assert old_container.get('Config.Cmd') == ['-d', '1']
         assert 'FOO=1' in old_container.get('Config.Env')
-        assert old_container.name == 'composetest_db_1'
+        assert old_container.name.startswith('composetest_db_')
         service.start_container(old_container)
         old_container.inspect()  # reload volume data
         volume_path = old_container.get_mount('/etc')['Source']
@@ -503,7 +522,7 @@ class ServiceTest(DockerClientTestCase):
         assert new_container.get('Config.Entrypoint') == ['top']
         assert new_container.get('Config.Cmd') == ['-d', '1']
         assert 'FOO=2' in new_container.get('Config.Env')
-        assert new_container.name == 'composetest_db_1'
+        assert new_container.name.startswith('composetest_db_')
         assert new_container.get_mount('/etc')['Source'] == volume_path
         if not is_cluster(self.client):
             assert (
@@ -679,8 +698,8 @@ class ServiceTest(DockerClientTestCase):
             new_container, = service.execute_convergence_plan(
                 ConvergencePlan('recreate', [old_container]))
 
-        mock_log.warn.assert_called_once_with(mock.ANY)
-        _, args, kwargs = mock_log.warn.mock_calls[0]
+        mock_log.warning.assert_called_once_with(mock.ANY)
+        _, args, kwargs = mock_log.warning.mock_calls[0]
         assert "Service \"db\" is using volume \"/data\" from the previous container" in args[0]
 
         assert [mount['Destination'] for mount in new_container.get('Mounts')] == ['/data']
@@ -836,13 +855,13 @@ class ServiceTest(DockerClientTestCase):
         db = self.create_service('db')
         web = self.create_service('web', links=[(db, None)])
 
-        create_and_start_container(db)
-        create_and_start_container(db)
+        db1 = create_and_start_container(db)
+        db2 = create_and_start_container(db)
         create_and_start_container(web)
 
         assert set(get_links(web.containers()[0])) == set([
-            'composetest_db_1', 'db_1',
-            'composetest_db_2', 'db_2',
+            db1.name, db1.name_without_project,
+            db2.name, db2.name_without_project,
             'db'
         ])
 
@@ -851,30 +870,33 @@ class ServiceTest(DockerClientTestCase):
         db = self.create_service('db')
         web = self.create_service('web', links=[(db, 'custom_link_name')])
 
-        create_and_start_container(db)
-        create_and_start_container(db)
+        db1 = create_and_start_container(db)
+        db2 = create_and_start_container(db)
         create_and_start_container(web)
 
         assert set(get_links(web.containers()[0])) == set([
-            'composetest_db_1', 'db_1',
-            'composetest_db_2', 'db_2',
+            db1.name, db1.name_without_project,
+            db2.name, db2.name_without_project,
             'custom_link_name'
         ])
 
     @no_cluster('No legacy links support in Swarm')
     def test_start_container_with_external_links(self):
         db = self.create_service('db')
-        web = self.create_service('web', external_links=['composetest_db_1',
-                                                         'composetest_db_2',
-                                                         'composetest_db_3:db_3'])
+        db_ctnrs = [create_and_start_container(db) for _ in range(3)]
+        web = self.create_service(
+            'web', external_links=[
+                db_ctnrs[0].name,
+                db_ctnrs[1].name,
+                '{}:db_3'.format(db_ctnrs[2].name)
+            ]
+        )
 
-        for _ in range(3):
-            create_and_start_container(db)
         create_and_start_container(web)
 
         assert set(get_links(web.containers()[0])) == set([
-            'composetest_db_1',
-            'composetest_db_2',
+            db_ctnrs[0].name,
+            db_ctnrs[1].name,
             'db_3'
         ])
 
@@ -892,14 +914,14 @@ class ServiceTest(DockerClientTestCase):
     def test_start_one_off_container_creates_links_to_its_own_service(self):
         db = self.create_service('db')
 
-        create_and_start_container(db)
-        create_and_start_container(db)
+        db1 = create_and_start_container(db)
+        db2 = create_and_start_container(db)
 
         c = create_and_start_container(db, one_off=OneOffFilter.only)
 
         assert set(get_links(c)) == set([
-            'composetest_db_1', 'db_1',
-            'composetest_db_2', 'db_2',
+            db1.name, db1.name_without_project,
+            db2.name, db2.name_without_project,
             'db'
         ])
 
@@ -946,6 +968,43 @@ class ServiceTest(DockerClientTestCase):
 
         assert self.client.inspect_image('composetest_web')
 
+    def test_build_cli(self):
+        base_dir = tempfile.mkdtemp()
+        self.addCleanup(shutil.rmtree, base_dir)
+
+        with open(os.path.join(base_dir, 'Dockerfile'), 'w') as f:
+            f.write("FROM busybox\n")
+
+        service = self.create_service('web',
+                                      build={'context': base_dir},
+                                      environment={
+                                          'COMPOSE_DOCKER_CLI_BUILD': '1',
+                                          'DOCKER_BUILDKIT': '1',
+                                      })
+        service.build(cli=True)
+        self.addCleanup(self.client.remove_image, service.image_name)
+        assert self.client.inspect_image('composetest_web')
+
+    def test_up_build_cli(self):
+        base_dir = tempfile.mkdtemp()
+        self.addCleanup(shutil.rmtree, base_dir)
+
+        with open(os.path.join(base_dir, 'Dockerfile'), 'w') as f:
+            f.write("FROM busybox\n")
+
+        web = self.create_service('web',
+                                  build={'context': base_dir},
+                                  environment={
+                                      'COMPOSE_DOCKER_CLI_BUILD': '1',
+                                      'DOCKER_BUILDKIT': '1',
+                                  })
+        project = Project('composetest', [web], self.client)
+        project.up(do_build=BuildAction.force)
+
+        containers = project.containers(['web'])
+        assert len(containers) == 1
+        assert containers[0].name.startswith('composetest_web_')
+
     def test_build_non_ascii_filename(self):
         base_dir = tempfile.mkdtemp()
         self.addCleanup(shutil.rmtree, base_dir)
@@ -1137,6 +1196,21 @@ class ServiceTest(DockerClientTestCase):
         service.build()
         assert service.image()
 
+    def test_build_with_illegal_leading_chars(self):
+        base_dir = tempfile.mkdtemp()
+        self.addCleanup(shutil.rmtree, base_dir)
+        with open(os.path.join(base_dir, 'Dockerfile'), 'w') as f:
+            f.write('FROM busybox\nRUN echo "Embodiment of Scarlet Devil"\n')
+        service = Service(
+            'build_leading_slug', client=self.client,
+            project='___-composetest', build={
+                'context': text_type(base_dir)
+            }
+        )
+        assert service.image_name == 'composetest_build_leading_slug'
+        service.build()
+        assert service.image()
+
     def test_start_container_stays_unprivileged(self):
         service = self.create_service('web')
         container = create_and_start_container(service).inspect()
@@ -1198,9 +1272,8 @@ class ServiceTest(DockerClientTestCase):
         # })
 
     def test_create_with_image_id(self):
-        # Get image id for the current busybox:latest
         pull_busybox(self.client)
-        image_id = self.client.inspect_image('busybox:latest')['Id'][:12]
+        image_id = self.client.inspect_image(BUSYBOX_IMAGE_WITH_TAG)['Id'][:12]
         service = self.create_service('foo', image=image_id)
         service.create_container()
 
@@ -1234,17 +1307,15 @@ class ServiceTest(DockerClientTestCase):
         test that those containers are restarted and not removed/recreated.
         """
         service = self.create_service('web')
-        next_number = service._next_container_number()
-        valid_numbers = [next_number, next_number + 1]
-        service.create_container(number=next_number)
-        service.create_container(number=next_number + 1)
+        service.create_container(number=1)
+        service.create_container(number=2)
 
         ParallelStreamWriter.instance = None
         with mock.patch('sys.stderr', new_callable=StringIO) as mock_stderr:
             service.scale(2)
         for container in service.containers():
             assert container.is_running
-            assert container.number in valid_numbers
+            assert container.number in [1, 2]
 
         captured_output = mock_stderr.getvalue()
         assert 'Creating' not in captured_output
@@ -1295,10 +1366,8 @@ class ServiceTest(DockerClientTestCase):
 
         assert len(service.containers()) == 1
         assert service.containers()[0].is_running
-        assert (
-            "ERROR: for composetest_web_2  Cannot create container for service"
-            " web: Boom" in mock_stderr.getvalue()
-        )
+        assert "ERROR: for composetest_web_" in mock_stderr.getvalue()
+        assert "Cannot create container for service web: Boom" in mock_stderr.getvalue()
 
     def test_scale_with_unexpected_exception(self):
         """Test that when scaling if the API returns an error, that is not of type
@@ -1352,7 +1421,7 @@ class ServiceTest(DockerClientTestCase):
         with pytest.raises(OperationFailedError):
             service.scale(3)
 
-        captured_output = mock_log.warn.call_args[0][0]
+        captured_output = mock_log.warning.call_args[0][0]
 
         assert len(service.containers()) == 1
         assert "Remove the custom name to scale the service." in captured_output
@@ -1565,16 +1634,17 @@ class ServiceTest(DockerClientTestCase):
         }
 
         compose_labels = {
-            LABEL_CONTAINER_NUMBER: '1',
             LABEL_ONE_OFF: 'False',
             LABEL_PROJECT: 'composetest',
             LABEL_SERVICE: 'web',
             LABEL_VERSION: __version__,
+            LABEL_CONTAINER_NUMBER: '1'
         }
         expected = dict(labels_dict, **compose_labels)
 
         service = self.create_service('web', labels=labels_dict)
-        labels = create_and_start_container(service).labels.items()
+        ctnr = create_and_start_container(service)
+        labels = ctnr.labels.items()
         for pair in expected.items():
             assert pair in labels
 
@@ -1640,7 +1710,7 @@ class ServiceTest(DockerClientTestCase):
     def test_duplicate_containers(self):
         service = self.create_service('web')
 
-        options = service._get_container_create_options({}, 1)
+        options = service._get_container_create_options({}, service._next_container_number())
         original = Container.create(service.client, **options)
 
         assert set(service.containers(stopped=True)) == set([original])
