@@ -16,18 +16,18 @@ class VerboseProxyTestCase(unittest.TestCase):
             ("arg1", True),
             {'key': 'value'})
 
-        self.assertEqual(expected, actual)
+        assert expected == actual
 
     def test_format_return_sequence(self):
         expected = "(list with 10 items)"
         actual = verbose_proxy.format_return(list(range(10)), 2)
-        self.assertEqual(expected, actual)
+        assert expected == actual
 
     def test_format_return(self):
         expected = repr({'Id': 'ok'})
         actual = verbose_proxy.format_return({'Id': 'ok'}, 2)
-        self.assertEqual(expected, actual)
+        assert expected == actual
 
     def test_format_return_no_result(self):
         actual = verbose_proxy.format_return(None, 2)
-        self.assertEqual(None, actual)
+        assert actual is None
