@@ -3,6 +3,8 @@ from __future__ import absolute_import
 from __future__ import unicode_literals
 
 import datetime
+import os
+import tempfile
 
 import docker
 import pytest
@@ -10,14 +12,19 @@ from docker.errors import NotFound
 
 from .. import mock
 from .. import unittest
+from ..helpers import BUSYBOX_IMAGE_WITH_TAG
+from compose.config import ConfigurationError
 from compose.config.config import Config
 from compose.config.types import VolumeFromSpec
 from compose.const import COMPOSEFILE_V1 as V1
 from compose.const import COMPOSEFILE_V2_0 as V2_0
 from compose.const import COMPOSEFILE_V2_4 as V2_4
+from compose.const import COMPOSEFILE_V3_7 as V3_7
+from compose.const import DEFAULT_TIMEOUT
 from compose.const import LABEL_SERVICE
 from compose.container import Container
 from compose.errors import OperationFailedError
+from compose.project import get_secrets
 from compose.project import NoSuchService
 from compose.project import Project
 from compose.project import ProjectError
@@ -29,6 +36,7 @@ class ProjectTest(unittest.TestCase):
     def setUp(self):
         self.mock_client = mock.create_autospec(docker.APIClient)
         self.mock_client._general_configs = {}
+        self.mock_client.api_version = docker.constants.DEFAULT_DOCKER_API_VERSION
 
     def test_from_config_v1(self):
         config = Config(
@@ -36,11 +44,11 @@ class ProjectTest(unittest.TestCase):
             services=[
                 {
                     'name': 'web',
-                    'image': 'busybox:latest',
+                    'image': BUSYBOX_IMAGE_WITH_TAG,
                 },
                 {
                     'name': 'db',
-                    'image': 'busybox:latest',
+                    'image': BUSYBOX_IMAGE_WITH_TAG,
                 },
             ],
             networks=None,
@@ -55,22 +63,23 @@ class ProjectTest(unittest.TestCase):
         )
         assert len(project.services) == 2
         assert project.get_service('web').name == 'web'
-        assert project.get_service('web').options['image'] == 'busybox:latest'
+        assert project.get_service('web').options['image'] == BUSYBOX_IMAGE_WITH_TAG
         assert project.get_service('db').name == 'db'
-        assert project.get_service('db').options['image'] == 'busybox:latest'
+        assert project.get_service('db').options['image'] == BUSYBOX_IMAGE_WITH_TAG
         assert not project.networks.use_networking
 
+    @mock.patch('compose.network.Network.true_name', lambda n: n.full_name)
     def test_from_config_v2(self):
         config = Config(
             version=V2_0,
             services=[
                 {
                     'name': 'web',
-                    'image': 'busybox:latest',
+                    'image': BUSYBOX_IMAGE_WITH_TAG,
                 },
                 {
                     'name': 'db',
-                    'image': 'busybox:latest',
+                    'image': BUSYBOX_IMAGE_WITH_TAG,
                 },
             ],
             networks=None,
@@ -87,7 +96,7 @@ class ProjectTest(unittest.TestCase):
             project='composetest',
             name='web',
             client=None,
-            image="busybox:latest",
+            image=BUSYBOX_IMAGE_WITH_TAG,
         )
         project = Project('test', [web], None)
         assert project.get_service('web') == web
@@ -172,7 +181,7 @@ class ProjectTest(unittest.TestCase):
                 version=V2_0,
                 services=[{
                     'name': 'test',
-                    'image': 'busybox:latest',
+                    'image': BUSYBOX_IMAGE_WITH_TAG,
                     'volumes_from': [VolumeFromSpec('aaa', 'rw', 'container')]
                 }],
                 networks=None,
@@ -190,7 +199,7 @@ class ProjectTest(unittest.TestCase):
                 "Name": container_name,
                 "Names": [container_name],
                 "Id": container_name,
-                "Image": 'busybox:latest'
+                "Image": BUSYBOX_IMAGE_WITH_TAG
             }
         ]
         project = Project.from_config(
@@ -201,11 +210,11 @@ class ProjectTest(unittest.TestCase):
                 services=[
                     {
                         'name': 'vol',
-                        'image': 'busybox:latest'
+                        'image': BUSYBOX_IMAGE_WITH_TAG
                     },
                     {
                         'name': 'test',
-                        'image': 'busybox:latest',
+                        'image': BUSYBOX_IMAGE_WITH_TAG,
                         'volumes_from': [VolumeFromSpec('vol', 'rw', 'service')]
                     }
                 ],
@@ -217,6 +226,7 @@ class ProjectTest(unittest.TestCase):
         )
         assert project.get_service('test')._get_volumes_from() == [container_name + ":rw"]
 
+    @mock.patch('compose.network.Network.true_name', lambda n: n.full_name)
     def test_use_volumes_from_service_container(self):
         container_ids = ['aabbccddee', '12345']
 
@@ -228,11 +238,11 @@ class ProjectTest(unittest.TestCase):
                 services=[
                     {
                         'name': 'vol',
-                        'image': 'busybox:latest'
+                        'image': BUSYBOX_IMAGE_WITH_TAG
                     },
                     {
                         'name': 'test',
-                        'image': 'busybox:latest',
+                        'image': BUSYBOX_IMAGE_WITH_TAG,
                         'volumes_from': [VolumeFromSpec('vol', 'rw', 'service')]
                     }
                 ],
@@ -251,9 +261,10 @@ class ProjectTest(unittest.TestCase):
                 [container_ids[0] + ':rw']
             )
 
-    def test_events(self):
+    def test_events_legacy(self):
         services = [Service(name='web'), Service(name='db')]
         project = Project('test', services, self.mock_client)
+        self.mock_client.api_version = '1.21'
         self.mock_client.events.return_value = iter([
             {
                 'status': 'create',
@@ -359,6 +370,175 @@ class ProjectTest(unittest.TestCase):
             },
         ]
 
+    def test_events(self):
+        services = [Service(name='web'), Service(name='db')]
+        project = Project('test', services, self.mock_client)
+        self.mock_client.api_version = '1.35'
+        self.mock_client.events.return_value = iter([
+            {
+                'status': 'create',
+                'from': 'example/image',
+                'Type': 'container',
+                'Actor': {
+                    'ID': 'abcde',
+                    'Attributes': {
+                        'com.docker.compose.project': 'test',
+                        'com.docker.compose.service': 'web',
+                        'image': 'example/image',
+                        'name': 'test_web_1',
+                    }
+                },
+                'id': 'abcde',
+                'time': 1420092061,
+                'timeNano': 14200920610000002000,
+            },
+            {
+                'status': 'attach',
+                'from': 'example/image',
+                'Type': 'container',
+                'Actor': {
+                    'ID': 'abcde',
+                    'Attributes': {
+                        'com.docker.compose.project': 'test',
+                        'com.docker.compose.service': 'web',
+                        'image': 'example/image',
+                        'name': 'test_web_1',
+                    }
+                },
+                'id': 'abcde',
+                'time': 1420092061,
+                'timeNano': 14200920610000003000,
+            },
+            {
+                'status': 'create',
+                'from': 'example/other',
+                'Type': 'container',
+                'Actor': {
+                    'ID': 'bdbdbd',
+                    'Attributes': {
+                        'image': 'example/other',
+                        'name': 'shrewd_einstein',
+                    }
+                },
+                'id': 'bdbdbd',
+                'time': 1420092061,
+                'timeNano': 14200920610000005000,
+            },
+            {
+                'status': 'create',
+                'from': 'example/db',
+                'Type': 'container',
+                'Actor': {
+                    'ID': 'ababa',
+                    'Attributes': {
+                        'com.docker.compose.project': 'test',
+                        'com.docker.compose.service': 'db',
+                        'image': 'example/db',
+                        'name': 'test_db_1',
+                    }
+                },
+                'id': 'ababa',
+                'time': 1420092061,
+                'timeNano': 14200920610000004000,
+            },
+            {
+                'status': 'destroy',
+                'from': 'example/db',
+                'Type': 'container',
+                'Actor': {
+                    'ID': 'eeeee',
+                    'Attributes': {
+                        'com.docker.compose.project': 'test',
+                        'com.docker.compose.service': 'db',
+                        'image': 'example/db',
+                        'name': 'test_db_1',
+                    }
+                },
+                'id': 'eeeee',
+                'time': 1420092061,
+                'timeNano': 14200920610000004000,
+            },
+        ])
+
+        def dt_with_microseconds(dt, us):
+            return datetime.datetime.fromtimestamp(dt).replace(microsecond=us)
+
+        def get_container(cid):
+            if cid == 'eeeee':
+                raise NotFound(None, None, "oops")
+            if cid == 'abcde':
+                name = 'web'
+                labels = {LABEL_SERVICE: name}
+            elif cid == 'ababa':
+                name = 'db'
+                labels = {LABEL_SERVICE: name}
+            else:
+                labels = {}
+                name = ''
+            return {
+                'Id': cid,
+                'Config': {'Labels': labels},
+                'Name': '/project_%s_1' % name,
+            }
+
+        self.mock_client.inspect_container.side_effect = get_container
+
+        events = project.events()
+
+        events_list = list(events)
+        # Assert the return value is a generator
+        assert not list(events)
+        assert events_list == [
+            {
+                'type': 'container',
+                'service': 'web',
+                'action': 'create',
+                'id': 'abcde',
+                'attributes': {
+                    'name': 'test_web_1',
+                    'image': 'example/image',
+                },
+                'time': dt_with_microseconds(1420092061, 2),
+                'container': Container(None, get_container('abcde')),
+            },
+            {
+                'type': 'container',
+                'service': 'web',
+                'action': 'attach',
+                'id': 'abcde',
+                'attributes': {
+                    'name': 'test_web_1',
+                    'image': 'example/image',
+                },
+                'time': dt_with_microseconds(1420092061, 3),
+                'container': Container(None, get_container('abcde')),
+            },
+            {
+                'type': 'container',
+                'service': 'db',
+                'action': 'create',
+                'id': 'ababa',
+                'attributes': {
+                    'name': 'test_db_1',
+                    'image': 'example/db',
+                },
+                'time': dt_with_microseconds(1420092061, 4),
+                'container': Container(None, get_container('ababa')),
+            },
+            {
+                'type': 'container',
+                'service': 'db',
+                'action': 'destroy',
+                'id': 'eeeee',
+                'attributes': {
+                    'name': 'test_db_1',
+                    'image': 'example/db',
+                },
+                'time': dt_with_microseconds(1420092061, 4),
+                'container': None,
+            },
+        ]
+
     def test_net_unset(self):
         project = Project.from_config(
             name='test',
@@ -368,7 +548,7 @@ class ProjectTest(unittest.TestCase):
                 services=[
                     {
                         'name': 'test',
-                        'image': 'busybox:latest',
+                        'image': BUSYBOX_IMAGE_WITH_TAG,
                     }
                 ],
                 networks=None,
@@ -393,7 +573,7 @@ class ProjectTest(unittest.TestCase):
                 services=[
                     {
                         'name': 'test',
-                        'image': 'busybox:latest',
+                        'image': BUSYBOX_IMAGE_WITH_TAG,
                         'network_mode': 'container:aaa'
                     },
                 ],
@@ -413,7 +593,7 @@ class ProjectTest(unittest.TestCase):
                 "Name": container_name,
                 "Names": [container_name],
                 "Id": container_name,
-                "Image": 'busybox:latest'
+                "Image": BUSYBOX_IMAGE_WITH_TAG
             }
         ]
         project = Project.from_config(
@@ -424,11 +604,11 @@ class ProjectTest(unittest.TestCase):
                 services=[
                     {
                         'name': 'aaa',
-                        'image': 'busybox:latest'
+                        'image': BUSYBOX_IMAGE_WITH_TAG
                     },
                     {
                         'name': 'test',
-                        'image': 'busybox:latest',
+                        'image': BUSYBOX_IMAGE_WITH_TAG,
                         'network_mode': 'service:aaa'
                     },
                 ],
@@ -451,7 +631,7 @@ class ProjectTest(unittest.TestCase):
                 services=[
                     {
                         'name': 'foo',
-                        'image': 'busybox:latest'
+                        'image': BUSYBOX_IMAGE_WITH_TAG
                     },
                 ],
                 networks=None,
@@ -472,7 +652,7 @@ class ProjectTest(unittest.TestCase):
                 services=[
                     {
                         'name': 'foo',
-                        'image': 'busybox:latest',
+                        'image': BUSYBOX_IMAGE_WITH_TAG,
                         'networks': {'custom': None}
                     },
                 ],
@@ -487,9 +667,9 @@ class ProjectTest(unittest.TestCase):
 
     def test_container_without_name(self):
         self.mock_client.containers.return_value = [
-            {'Image': 'busybox:latest', 'Id': '1', 'Name': '1'},
-            {'Image': 'busybox:latest', 'Id': '2', 'Name': None},
-            {'Image': 'busybox:latest', 'Id': '3'},
+            {'Image': BUSYBOX_IMAGE_WITH_TAG, 'Id': '1', 'Name': '1'},
+            {'Image': BUSYBOX_IMAGE_WITH_TAG, 'Id': '2', 'Name': None},
+            {'Image': BUSYBOX_IMAGE_WITH_TAG, 'Id': '3'},
         ]
         self.mock_client.inspect_container.return_value = {
             'Id': '1',
@@ -506,7 +686,7 @@ class ProjectTest(unittest.TestCase):
                 version=V2_0,
                 services=[{
                     'name': 'web',
-                    'image': 'busybox:latest',
+                    'image': BUSYBOX_IMAGE_WITH_TAG,
                 }],
                 networks=None,
                 volumes=None,
@@ -524,7 +704,7 @@ class ProjectTest(unittest.TestCase):
                 version=V2_0,
                 services=[{
                     'name': 'web',
-                    'image': 'busybox:latest',
+                    'image': BUSYBOX_IMAGE_WITH_TAG,
                 }],
                 networks={'default': {}},
                 volumes={'data': {}},
@@ -536,7 +716,7 @@ class ProjectTest(unittest.TestCase):
         self.mock_client.remove_volume.side_effect = NotFound(None, None, 'oops')
 
         project.down(ImageType.all, True)
-        self.mock_client.remove_image.assert_called_once_with("busybox:latest")
+        self.mock_client.remove_image.assert_called_once_with(BUSYBOX_IMAGE_WITH_TAG)
 
     def test_no_warning_on_stop(self):
         self.mock_client.info.return_value = {'Swarm': {'LocalNodeState': 'active'}}
@@ -569,28 +749,56 @@ class ProjectTest(unittest.TestCase):
     def test_project_platform_value(self):
         service_config = {
             'name': 'web',
-            'image': 'busybox:latest',
+            'image': BUSYBOX_IMAGE_WITH_TAG,
         }
         config_data = Config(
             version=V2_4, services=[service_config], networks={}, volumes={}, secrets=None, configs=None
         )
 
         project = Project.from_config(name='test', client=self.mock_client, config_data=config_data)
-        assert project.get_service('web').options.get('platform') is None
+        assert project.get_service('web').platform is None
 
         project = Project.from_config(
             name='test', client=self.mock_client, config_data=config_data, default_platform='windows'
         )
-        assert project.get_service('web').options.get('platform') == 'windows'
+        assert project.get_service('web').platform == 'windows'
 
         service_config['platform'] = 'linux/s390x'
         project = Project.from_config(name='test', client=self.mock_client, config_data=config_data)
-        assert project.get_service('web').options.get('platform') == 'linux/s390x'
+        assert project.get_service('web').platform == 'linux/s390x'
 
         project = Project.from_config(
             name='test', client=self.mock_client, config_data=config_data, default_platform='windows'
         )
-        assert project.get_service('web').options.get('platform') == 'linux/s390x'
+        assert project.get_service('web').platform == 'linux/s390x'
+
+    def test_build_container_operation_with_timeout_func_does_not_mutate_options_with_timeout(self):
+        config_data = Config(
+            version=V3_7,
+            services=[
+                {'name': 'web', 'image': BUSYBOX_IMAGE_WITH_TAG},
+                {'name': 'db', 'image': BUSYBOX_IMAGE_WITH_TAG, 'stop_grace_period': '1s'},
+            ],
+            networks={}, volumes={}, secrets=None, configs=None,
+        )
+
+        project = Project.from_config(name='test', client=self.mock_client, config_data=config_data)
+
+        stop_op = project.build_container_operation_with_timeout_func('stop', options={})
+
+        web_container = mock.create_autospec(Container, service='web')
+        db_container = mock.create_autospec(Container, service='db')
+
+        # `stop_grace_period` is not set to 'web' service,
+        # then it is stopped with the default timeout.
+        stop_op(web_container)
+        web_container.stop.assert_called_once_with(timeout=DEFAULT_TIMEOUT)
+
+        # `stop_grace_period` is set to 'db' service,
+        # then it is stopped with the specified timeout and
+        # the value is not overridden by the previous function call.
+        stop_op(db_container)
+        db_container.stop.assert_called_once_with(timeout=1)
 
     @mock.patch('compose.parallel.ParallelStreamWriter._write_noansi')
     def test_error_parallel_pull(self, mock_write):
@@ -601,7 +809,7 @@ class ProjectTest(unittest.TestCase):
                 version=V2_0,
                 services=[{
                     'name': 'web',
-                    'image': 'busybox:latest',
+                    'image': BUSYBOX_IMAGE_WITH_TAG,
                 }],
                 networks=None,
                 volumes=None,
@@ -617,3 +825,104 @@ class ProjectTest(unittest.TestCase):
         self.mock_client.pull.side_effect = OperationFailedError(b'pull error')
         with pytest.raises(ProjectError):
             project.pull(parallel_pull=True)
+
+    def test_avoid_multiple_push(self):
+        service_config_latest = {'image': 'busybox:latest', 'build': '.'}
+        service_config_default = {'image': 'busybox', 'build': '.'}
+        service_config_sha = {
+            'image': 'busybox@sha256:38a203e1986cf79639cfb9b2e1d6e773de84002feea2d4eb006b52004ee8502d',
+            'build': '.'
+        }
+        svc1 = Service('busy1', **service_config_latest)
+        svc1_1 = Service('busy11', **service_config_latest)
+        svc2 = Service('busy2', **service_config_default)
+        svc2_1 = Service('busy21', **service_config_default)
+        svc3 = Service('busy3', **service_config_sha)
+        svc3_1 = Service('busy31', **service_config_sha)
+        project = Project(
+            'composetest', [svc1, svc1_1, svc2, svc2_1, svc3, svc3_1], self.mock_client
+        )
+        with mock.patch('compose.service.Service.push') as fake_push:
+            project.push()
+            assert fake_push.call_count == 2
+
+    def test_get_secrets_no_secret_def(self):
+        service = 'foo'
+        secret_source = 'bar'
+
+        secret_defs = mock.Mock()
+        secret_defs.get.return_value = None
+        secret = mock.Mock(source=secret_source)
+
+        with self.assertRaises(ConfigurationError):
+            get_secrets(service, [secret], secret_defs)
+
+    def test_get_secrets_external_warning(self):
+        service = 'foo'
+        secret_source = 'bar'
+
+        secret_def = mock.Mock()
+        secret_def.get.return_value = True
+
+        secret_defs = mock.Mock()
+        secret_defs.get.side_effect = secret_def
+        secret = mock.Mock(source=secret_source)
+
+        with mock.patch('compose.project.log') as mock_log:
+            get_secrets(service, [secret], secret_defs)
+
+        mock_log.warning.assert_called_with("Service \"{service}\" uses secret \"{secret}\" "
+                                            "which is external. External secrets are not available"
+                                            " to containers created by docker-compose."
+                                            .format(service=service, secret=secret_source))
+
+    def test_get_secrets_uid_gid_mode_warning(self):
+        service = 'foo'
+        secret_source = 'bar'
+
+        fd, filename_path = tempfile.mkstemp()
+        os.close(fd)
+        self.addCleanup(os.remove, filename_path)
+
+        def mock_get(key):
+            return {'external': False, 'file': filename_path}[key]
+
+        secret_def = mock.MagicMock()
+        secret_def.get = mock.MagicMock(side_effect=mock_get)
+
+        secret_defs = mock.Mock()
+        secret_defs.get.return_value = secret_def
+
+        secret = mock.Mock(uid=True, gid=True, mode=True, source=secret_source)
+
+        with mock.patch('compose.project.log') as mock_log:
+            get_secrets(service, [secret], secret_defs)
+
+        mock_log.warning.assert_called_with("Service \"{service}\" uses secret \"{secret}\" with uid, "
+                                            "gid, or mode. These fields are not supported by this "
+                                            "implementation of the Compose file"
+                                            .format(service=service, secret=secret_source))
+
+    def test_get_secrets_secret_file_warning(self):
+        service = 'foo'
+        secret_source = 'bar'
+        not_a_path = 'NOT_A_PATH'
+
+        def mock_get(key):
+            return {'external': False, 'file': not_a_path}[key]
+
+        secret_def = mock.MagicMock()
+        secret_def.get = mock.MagicMock(side_effect=mock_get)
+
+        secret_defs = mock.Mock()
+        secret_defs.get.return_value = secret_def
+
+        secret = mock.Mock(uid=False, gid=False, mode=False, source=secret_source)
+
+        with mock.patch('compose.project.log') as mock_log:
+            get_secrets(service, [secret], secret_defs)
+
+        mock_log.warning.assert_called_with("Service \"{service}\" uses an undefined secret file "
+                                            "\"{secret_file}\", the following file should be created "
+                                            "\"{secret_file}\""
+                                            .format(service=service, secret_file=not_a_path))
