@@ -5,11 +5,13 @@ import docker
 import pytest
 from docker.constants import DEFAULT_DOCKER_API_VERSION
 from docker.errors import APIError
+from docker.errors import ImageNotFound
 from docker.errors import NotFound
 
 from .. import mock
 from .. import unittest
 from compose.config.errors import DependencyError
+from compose.config.types import MountSpec
 from compose.config.types import ServicePort
 from compose.config.types import ServiceSecret
 from compose.config.types import VolumeFromSpec
@@ -20,6 +22,7 @@ from compose.const import LABEL_ONE_OFF
 from compose.const import LABEL_PROJECT
 from compose.const import LABEL_SERVICE
 from compose.const import SECRETS_PATH
+from compose.const import WINDOWS_LONGPATH_PREFIX
 from compose.container import Container
 from compose.errors import OperationFailedError
 from compose.parallel import ParallelStreamWriter
@@ -37,6 +40,7 @@ from compose.service import NeedsBuildError
 from compose.service import NetworkMode
 from compose.service import NoSuchImageError
 from compose.service import parse_repository_tag
+from compose.service import rewrite_build_path
 from compose.service import Service
 from compose.service import ServiceNetworkMode
 from compose.service import warn_on_masked_volume
@@ -316,19 +320,20 @@ class ServiceTest(unittest.TestCase):
         self.mock_client.inspect_image.return_value = {'Id': 'abcd'}
         prev_container = mock.Mock(
             id='ababab',
-            image_config={'ContainerConfig': {}})
+            image_config={'ContainerConfig': {}}
+        )
+        prev_container.full_slug = 'abcdefff1234'
         prev_container.get.return_value = None
 
         opts = service._get_container_create_options(
-            {},
-            1,
-            previous_container=prev_container)
+            {}, 1, previous_container=prev_container
+        )
 
         assert service.options['labels'] == labels
         assert service.options['environment'] == environment
 
         assert opts['labels'][LABEL_CONFIG_HASH] == \
-            '2524a06fcb3d781aa2c981fc40bcfa08013bb318e4273bfa388df22023e6f2aa'
+            '689149e6041a85f6fb4945a2146a497ed43c8a5cbd8991753d875b165f1b4de4'
         assert opts['environment'] == ['also=real']
 
     def test_get_container_create_options_sets_affinity_with_binds(self):
@@ -354,11 +359,13 @@ class ServiceTest(unittest.TestCase):
             }.get(key, None)
 
         prev_container.get.side_effect = container_get
+        prev_container.full_slug = 'abcdefff1234'
 
         opts = service._get_container_create_options(
             {},
             1,
-            previous_container=prev_container)
+            previous_container=prev_container
+        )
 
         assert opts['environment'] == ['affinity:container==ababab']
 
@@ -369,6 +376,7 @@ class ServiceTest(unittest.TestCase):
             id='ababab',
             image_config={'ContainerConfig': {}})
         prev_container.get.return_value = None
+        prev_container.full_slug = 'abcdefff1234'
 
         opts = service._get_container_create_options(
             {},
@@ -385,7 +393,7 @@ class ServiceTest(unittest.TestCase):
 
     @mock.patch('compose.service.Container', autospec=True)
     def test_get_container(self, mock_container_class):
-        container_dict = dict(Name='default_foo_2')
+        container_dict = dict(Name='default_foo_2_bdfa3ed91e2c')
         self.mock_client.containers.return_value = [container_dict]
         service = Service('foo', image='foo', client=self.mock_client)
 
@@ -445,9 +453,24 @@ class ServiceTest(unittest.TestCase):
         with pytest.raises(OperationFailedError):
             service.pull()
 
+    def test_pull_image_with_default_platform(self):
+        self.mock_client.api_version = '1.35'
+
+        service = Service(
+            'foo', client=self.mock_client, image='someimage:sometag',
+            default_platform='linux'
+        )
+        assert service.platform == 'linux'
+        service.pull()
+
+        assert self.mock_client.pull.call_count == 1
+        call_args = self.mock_client.pull.call_args
+        assert call_args[1]['platform'] == 'linux'
+
     @mock.patch('compose.service.Container', autospec=True)
     def test_recreate_container(self, _):
         mock_container = mock.create_autospec(Container)
+        mock_container.full_slug = 'abcdefff1234'
         service = Service('foo', client=self.mock_client, image='someimage')
         service.image = lambda: {'Id': 'abc123'}
         new_container = service.recreate_container(mock_container)
@@ -461,6 +484,7 @@ class ServiceTest(unittest.TestCase):
     @mock.patch('compose.service.Container', autospec=True)
     def test_recreate_container_with_timeout(self, _):
         mock_container = mock.create_autospec(Container)
+        mock_container.full_slug = 'abcdefff1234'
         self.mock_client.inspect_image.return_value = {'Id': 'abc123'}
         service = Service('foo', client=self.mock_client, image='someimage')
         service.recreate_container(mock_container, timeout=1)
@@ -492,8 +516,8 @@ class ServiceTest(unittest.TestCase):
 
         with mock.patch('compose.service.log', autospec=True) as mock_log:
             service.create_container()
-            assert mock_log.warn.called
-            _, args, _ = mock_log.warn.mock_calls[0]
+            assert mock_log.warning.called
+            _, args, _ = mock_log.warning.mock_calls[0]
             assert 'was built because it did not already exist' in args[0]
 
         assert self.mock_client.build.call_count == 1
@@ -522,7 +546,7 @@ class ServiceTest(unittest.TestCase):
         with mock.patch('compose.service.log', autospec=True) as mock_log:
             service.ensure_image_exists(do_build=BuildAction.force)
 
-        assert not mock_log.warn.called
+        assert not mock_log.warning.called
         assert self.mock_client.build.call_count == 1
         self.mock_client.build.call_args[1]['tag'] == 'default_foo'
 
@@ -537,7 +561,7 @@ class ServiceTest(unittest.TestCase):
         assert self.mock_client.build.call_count == 1
         assert not self.mock_client.build.call_args[1]['pull']
 
-    def test_build_does_with_platform(self):
+    def test_build_with_platform(self):
         self.mock_client.api_version = '1.35'
         self.mock_client.build.return_value = [
             b'{"stream": "Successfully built 12345"}',
@@ -550,6 +574,47 @@ class ServiceTest(unittest.TestCase):
         call_args = self.mock_client.build.call_args
         assert call_args[1]['platform'] == 'linux'
 
+    def test_build_with_default_platform(self):
+        self.mock_client.api_version = '1.35'
+        self.mock_client.build.return_value = [
+            b'{"stream": "Successfully built 12345"}',
+        ]
+
+        service = Service(
+            'foo', client=self.mock_client, build={'context': '.'},
+            default_platform='linux'
+        )
+        assert service.platform == 'linux'
+        service.build()
+
+        assert self.mock_client.build.call_count == 1
+        call_args = self.mock_client.build.call_args
+        assert call_args[1]['platform'] == 'linux'
+
+    def test_service_platform_precedence(self):
+        self.mock_client.api_version = '1.35'
+
+        service = Service(
+            'foo', client=self.mock_client, platform='linux/arm',
+            default_platform='osx'
+        )
+        assert service.platform == 'linux/arm'
+
+    def test_service_ignore_default_platform_with_unsupported_api(self):
+        self.mock_client.api_version = '1.32'
+        self.mock_client.build.return_value = [
+            b'{"stream": "Successfully built 12345"}',
+        ]
+
+        service = Service(
+            'foo', client=self.mock_client, default_platform='windows', build={'context': '.'}
+        )
+        assert service.platform is None
+        service.build()
+        assert self.mock_client.build.call_count == 1
+        call_args = self.mock_client.build.call_args
+        assert call_args[1]['platform'] is None
+
     def test_build_with_override_build_args(self):
         self.mock_client.build.return_value = [
             b'{"stream": "Successfully built 12345"}',
@@ -611,6 +676,7 @@ class ServiceTest(unittest.TestCase):
             'options': {'image': 'example.com/foo'},
             'links': [('one', 'one')],
             'net': 'other',
+            'secrets': [],
             'networks': {'default': None},
             'volumes_from': [('two', 'rw')],
         }
@@ -633,6 +699,7 @@ class ServiceTest(unittest.TestCase):
             'options': {'image': 'example.com/foo'},
             'links': [],
             'networks': {},
+            'secrets': [],
             'net': 'aaabbb',
             'volumes_from': [],
         }
@@ -645,17 +712,19 @@ class ServiceTest(unittest.TestCase):
             image='example.com/foo',
             client=self.mock_client,
             network_mode=NetworkMode('bridge'),
-            networks={'bridge': {}},
+            networks={'bridge': {}, 'net2': {}},
             links=[(Service('one', client=self.mock_client), 'one')],
-            volumes_from=[VolumeFromSpec(Service('two', client=self.mock_client), 'rw', 'service')]
+            volumes_from=[VolumeFromSpec(Service('two', client=self.mock_client), 'rw', 'service')],
+            volumes=[VolumeSpec('/ext', '/int', 'ro')],
+            build={'context': 'some/random/path'},
         )
         config_hash = service.config_hash
 
         for api_version in set(API_VERSIONS.values()):
             self.mock_client.api_version = api_version
-            assert service._get_container_create_options({}, 1)['labels'][LABEL_CONFIG_HASH] == (
-                config_hash
-            )
+            assert service._get_container_create_options(
+                {}, 1
+            )['labels'][LABEL_CONFIG_HASH] == config_hash
 
     def test_remove_image_none(self):
         web = Service('web', image='example', client=self.mock_client)
@@ -689,6 +758,13 @@ class ServiceTest(unittest.TestCase):
         mock_log.error.assert_called_once_with(
             "Failed to remove image for service %s: %s", web.name, error)
 
+    def test_remove_non_existing_image(self):
+        self.mock_client.remove_image.side_effect = ImageNotFound('image not found')
+        web = Service('web', image='example', client=self.mock_client)
+        with mock.patch('compose.service.log', autospec=True) as mock_log:
+            assert not web.remove_image(ImageType.all)
+        mock_log.warning.assert_called_once_with("Image %s not found.", web.image_name)
+
     def test_specifies_host_port_with_no_ports(self):
         service = Service(
             'foo',
@@ -752,7 +828,7 @@ class ServiceTest(unittest.TestCase):
         assert service.specifies_host_port()
 
     def test_image_name_from_config(self):
-        image_name = 'example/web:latest'
+        image_name = 'example/web:mytag'
         service = Service('foo', image=image_name)
         assert service.image_name == image_name
 
@@ -771,13 +847,13 @@ class ServiceTest(unittest.TestCase):
             ports=["8080:80"])
 
         service.scale(0)
-        assert not mock_log.warn.called
+        assert not mock_log.warning.called
 
         service.scale(1)
-        assert not mock_log.warn.called
+        assert not mock_log.warning.called
 
         service.scale(2)
-        mock_log.warn.assert_called_once_with(
+        mock_log.warning.assert_called_once_with(
             'The "{}" service specifies a port on the host. If multiple containers '
             'for this service are created on a single host, the port will clash.'.format(name))
 
@@ -955,6 +1031,41 @@ class ServiceTest(unittest.TestCase):
 
         assert service.create_container().id == 'new_cont_id'
 
+    def test_build_volume_options_duplicate_binds(self):
+        self.mock_client.api_version = '1.29'  # Trigger 3.2 format workaround
+        service = Service('foo', client=self.mock_client)
+        ctnr_opts, override_opts = service._build_container_volume_options(
+            previous_container=None,
+            container_options={
+                'volumes': [
+                    MountSpec.parse({'source': 'vol', 'target': '/data', 'type': 'volume'}),
+                    VolumeSpec.parse('vol:/data:rw'),
+                ],
+                'environment': {},
+            },
+            override_options={},
+        )
+        assert 'binds' in override_opts
+        assert len(override_opts['binds']) == 1
+        assert override_opts['binds'][0] == 'vol:/data:rw'
+
+    def test_volumes_order_is_preserved(self):
+        service = Service('foo', client=self.mock_client)
+        volumes = [
+            VolumeSpec.parse(cfg) for cfg in [
+                '/v{0}:/v{0}:rw'.format(i) for i in range(6)
+            ]
+        ]
+        ctnr_opts, override_opts = service._build_container_volume_options(
+            previous_container=None,
+            container_options={
+                'volumes': volumes,
+                'environment': {},
+            },
+            override_options={},
+        )
+        assert override_opts['binds'] == [vol.repr() for vol in volumes]
+
 
 class TestServiceNetwork(unittest.TestCase):
     def setUp(self):
@@ -1223,10 +1334,8 @@ class ServiceVolumesTest(unittest.TestCase):
             number=1,
         )
 
-        assert set(self.mock_client.create_host_config.call_args[1]['binds']) == set([
-            '/host/path:/data1:rw',
-            '/host/path:/data2:rw',
-        ])
+        assert set(self.mock_client.create_host_config.call_args[1]['binds']) == {'/host/path:/data1:rw',
+                                                                                  '/host/path:/data2:rw'}
 
     def test_get_container_create_options_with_different_host_path_in_container_json(self):
         service = Service(
@@ -1280,7 +1389,7 @@ class ServiceVolumesTest(unittest.TestCase):
         with mock.patch('compose.service.log', autospec=True) as mock_log:
             warn_on_masked_volume(volumes_option, container_volumes, service)
 
-        assert not mock_log.warn.called
+        assert not mock_log.warning.called
 
     def test_warn_on_masked_volume_when_masked(self):
         volumes_option = [VolumeSpec('/home/user', '/path', 'rw')]
@@ -1293,7 +1402,7 @@ class ServiceVolumesTest(unittest.TestCase):
         with mock.patch('compose.service.log', autospec=True) as mock_log:
             warn_on_masked_volume(volumes_option, container_volumes, service)
 
-        mock_log.warn.assert_called_once_with(mock.ANY)
+        mock_log.warning.assert_called_once_with(mock.ANY)
 
     def test_warn_on_masked_no_warning_with_same_path(self):
         volumes_option = [VolumeSpec('/home/user', '/path', 'rw')]
@@ -1303,7 +1412,7 @@ class ServiceVolumesTest(unittest.TestCase):
         with mock.patch('compose.service.log', autospec=True) as mock_log:
             warn_on_masked_volume(volumes_option, container_volumes, service)
 
-        assert not mock_log.warn.called
+        assert not mock_log.warning.called
 
     def test_warn_on_masked_no_warning_with_container_only_option(self):
         volumes_option = [VolumeSpec(None, '/path', 'rw')]
@@ -1315,7 +1424,7 @@ class ServiceVolumesTest(unittest.TestCase):
         with mock.patch('compose.service.log', autospec=True) as mock_log:
             warn_on_masked_volume(volumes_option, container_volumes, service)
 
-        assert not mock_log.warn.called
+        assert not mock_log.warning.called
 
     def test_create_with_special_volume_mode(self):
         self.mock_client.inspect_image.return_value = {'Id': 'imageid'}
@@ -1387,3 +1496,28 @@ class ServiceSecretTest(unittest.TestCase):
 
         assert volumes[0].source == secret1['file']
         assert volumes[0].target == '{}/{}'.format(SECRETS_PATH, secret1['secret'].source)
+
+
+class RewriteBuildPathTest(unittest.TestCase):
+    @mock.patch('compose.service.IS_WINDOWS_PLATFORM', True)
+    def test_rewrite_url_no_prefix(self):
+        urls = [
+            'http://test.com',
+            'https://test.com',
+            'git://test.com',
+            'github.com/test/test',
+            'git@test.com',
+        ]
+        for u in urls:
+            assert rewrite_build_path(u) == u
+
+    @mock.patch('compose.service.IS_WINDOWS_PLATFORM', True)
+    def test_rewrite_windows_path(self):
+        assert rewrite_build_path('C:\\context') == WINDOWS_LONGPATH_PREFIX + 'C:\\context'
+        assert rewrite_build_path(
+            rewrite_build_path('C:\\context')
+        ) == rewrite_build_path('C:\\context')
+
+    @mock.patch('compose.service.IS_WINDOWS_PLATFORM', False)
+    def test_rewrite_unix_path(self):
+        assert rewrite_build_path('/context') == '/context'
