@@ -10,7 +10,7 @@ from tests import mock
 
 @pytest.fixture
 def mock_client():
-    return mock.create_autospec(docker.Client)
+    return mock.create_autospec(docker.APIClient)
 
 
 class TestVolume(object):
@@ -21,6 +21,6 @@ class TestVolume(object):
         mock_client.remove_volume.assert_called_once_with('foo_project')
 
     def test_remove_external_volume(self, mock_client):
-        vol = volume.Volume(mock_client, 'foo', 'project', external_name='data')
+        vol = volume.Volume(mock_client, 'foo', 'project', external=True)
         vol.remove()
         assert not mock_client.remove_volume.called
